#ifndef __COVSUM_H__
#define __COVSUM_H__

#include "ggpr.h"

#define NBLOCKS 512
#define NBLOCKSX 16
#define NBLOCKSY 16

/*
 * This is the static version of the covSum.m function. Currently it
 * calcualtes covariance matrix as a sum of covSEard and covNoise
 * functions. Additionaly, it returns derivatives along
 * hyperparameters.
 *
 * Input parameters:
 *   loghyper - hyperparameters
 *   D - size of loghyper
 *   N - X length
 *   z - hyperparameter coordinate, iff 0 then calculate cov. matrix
 *
 * The function uses static matrices defined in the ggpr.h header
 */
void covSum( real *loghyper, int D, int M, size_t NinDim, size_t Mtd, int z );

/*
 * Kernel declarations
 */

/* 
 *  This kernel creates the covar. matrix  (covSEard + covNoise)
 */
__global__ void calcCovarMatrix( real *K, size_t pitch, \
				 real *X, size_t pitchx, \
				 int N, size_t Nin, \
				 real s2, real sf2, real *ell \
				 );

/* 
 * This kernel creates the covar. matrix  (covSEard)
 */
__global__ void calcCovarMatrixSEARD( real *K, size_t pitch, \
				      real *X, size_t pitchx, \
				      int N, size_t Nin, \
				      real sf2, real *ell \
				      );

/*
 * Calculate derivative matrix for covSEard function z = 1
 */
__global__ void calcDerMatrix( real *A, size_t pitchA, \
			       real *K, size_t pitchK, \
			       real *X, size_t pitchx, \
			       int N, int z, real *ell \
			       );

__global__ void timesTwoCopy(real *A, size_t pitchA, real *K, size_t pitchK, int N);

/* 
 *  This kernel creates the covar. matrix  (covSEard + covNoise)
 */
__global__ void calcDerMatrixNoise(real *A, size_t pitchA, int N, real s2);

void dumpVariable (
  const char *name,
  double *d_var,
  size_t  pitch,
  size_t  nr,
  size_t  nc
  );

#endif
