#include "covSum.h"

/* 
 *  This kernel creates the covar. matrix  (covSEard + covNoise)
 */
__global__ void calcCovarMatrix(real *K, size_t pitch, real *X, size_t pitchx, int N, size_t Nin, real s2, real sf2, real *ell)
{
  int i = __umul24(blockIdx.x, blockDim.x) + threadIdx.x;
  int j = __umul24(blockIdx.y, blockDim.y) + threadIdx.y;
  int k;
  real dist;
  real *R;
  real *Xr;
  real t;


  R = (real *)((char*)K + i * pitch);
  if (i < N && j < N)
    {
      dist = 0;
      for (k = 0; k < Nin; k++) 
	{ 
	  Xr = (real *)((char*)X + k * pitchx); 
	  t = (Xr[i]-Xr[j])/ell[k]; 
	  dist += t*t;
	} 

      if (i == j)
	R[j] = sf2*exp(-dist/2) + s2;
      else 
	R[j] = sf2*exp(-dist/2);
    }
}

/* 
 * This kernel creates the covar. matrix  (covSEard)
 */
__global__ void calcCovarMatrixSEARD(real *K, size_t pitch, real *X, size_t pitchx, int N, size_t Nin, real sf2, real *ell)
{

  int i = __umul24(blockIdx.x, blockDim.x) + threadIdx.x;
  int j = __umul24(blockIdx.y, blockDim.y) + threadIdx.y;
  int k;
  real dist;
  real *R;
  real *Xr;
  real t;

  R = (real *)((char*)K + i * pitch);
  if (i < N && j < N)
    {
      dist = 0;
      for (k = 0; k < Nin; k++)
	{
	  Xr = (real *)((char*)X + k * pitchx);
	  t = (Xr[i]-Xr[j])/ell[k];
	  dist += t*t;
	}
      R[j] = sf2*exp(-dist/2);
    }
}

/*
 * Calculate derivative matrix for covSEard function z = 1
 */
__global__ void calcDerMatrix(real *A, size_t pitchA, real *K, size_t pitchK, real *X, size_t pitchx, int N, int z, real *ell)
{
  int i = blockIdx.x * blockDim.x + threadIdx.x;
  int j = blockIdx.y * blockDim.y + threadIdx.y;
  real dist;
  real *Xr;
  real *rK;
  real *rA;
  
  rK = (real *)((char*)K + i * pitchK);
  rA = (real *)((char*)A + i * pitchA);

  if (i < N && j < N)
    {
      Xr = (real *)((char*)X + z * pitchx);
      dist = (Xr[i]-Xr[j])/ell[z];
      rA[j] = rK[j] * dist*dist;
    }
}


__global__ void timesTwoCopy(real *A, size_t pitchA, real *K, size_t pitchK, int N)
{
  int i = blockIdx.x * blockDim.x + threadIdx.x;
  int j = blockIdx.y * blockDim.y + threadIdx.y;
  real *rK;
  real *rA;
  
  rK = (real *)((char*)K + i * pitchK);
  rA = (real *)((char*)A + i * pitchA);

  if (i < N && j < N)
    {
      rA[j] = 2*rK[j];
    }
}

/* 
 *  This kernel creates the covar. matrix  (covSEard + covNoise)
 */
__global__ void calcDerMatrixNoise(real *A, size_t pitchA, int N, real s2)
{
  int i = __umul24(blockIdx.x, blockDim.x) + threadIdx.x;
  real *R;
  R = (real *)((char*)A + i * pitchA);

  if (i < N)
    {
	R[i] = 2*s2;
    }
}
