function t = toy1_cov ( nData ) 

    fprintf ( 'Testing %04d elements ...\n', nData );
    
    load ../data/example_data5000

    yeq = example_train_data.yeq; 
    ueq = example_train_data.ueq; 
    utrain = example_train_data.utrain(1:nData); 
    xtrain = example_train_data.xtrain(1:nData); 
    ytrain = example_train_data.ytrain(1:nData); 

    % build training data (delayed outputs y first) 
    input  = [yeq ueq; xtrain utrain]; 
    target = [yeq; ytrain]; 

    % Select a GPU device
    gpudev = gpuDevice();
    % Reset the GPU device and clear the allocated memory
    reset(gpudev);
    % Move the input data to the GPU
    ginput = gpuArray ( input );

    % covariance function: SE + white noise 
    covfunc = {'covSEard','covNoise'}; 

    % to get the number of hyperparameters you could use: feval(covfunc{:})
    D = size(input,2); 

    profile on
    profile clear
    
    % training 
    for i=1:100
        logtheta0 = -rand(D+2,1); 
        C = covSum ( covfunc, logtheta0, ginput );
        % Fetch information from GPU device
        gpudev = gpuDevice();
        fprintf ( 'free mem: %d\n', gpudev.FreeMemory );
    end

    % save the profiling data
    s = profile('info');
    % get the timing from profiling data ... we do not know where `covSum`
    % is placed, and the location depends on the momentary state of the
    % profiler
    names = { s.FunctionTable.FunctionName };
    bs    = strcmp ( names, 'covSum' );
    s.FunctionTable(bs).FunctionName
    t = s.FunctionTable(bs).TotalTime;
    profile off

